@@ -10,6 +10,7 @@ from weakref import WeakSet
 import threading
 import copy
 import sys
+from threading import Lock
 
 
 class DataSocket:
@@ -22,8 +23,10 @@ class DataSocket:
         self._socket = context.socket(type)
         self._debug = debug
         # store these as wekrefs so that circular refs dont prevent garbage collection
-        self._java_objects = WeakSet()
+        self._java_objects = set()
         self._port = port
+        self._close_lock = Lock()
+        self._closed = False
         if type == zmq.PUSH:
             if debug:
                 print("binding {}".format(port))
@@ -172,15 +175,23 @@ class DataSocket:
         if "type" in response and response["type"] == "exception":
             raise Exception(response["value"])
 
+    def __del__(self):
+        self.close() # make sure it closes properly
+
     def close(self):
-        for java_object in self._java_objects:
-            java_object._close()
-        self._socket.close()
-        while not self._socket.closed:
-            time.sleep(0.01)
-        self._socket = None
-        if self._debug:
-            print('closed socket {}'.format(self._port))
+        with self._close_lock:
+            if not self._closed:
+                for java_object in self._java_objects:
+                    java_object._close()
+                    del java_object #potentially redundant, trying to fix closing race condition
+                self._java_objects = None
+                self._socket.close()
+                while not self._socket.closed:
+                    time.sleep(0.01)
+                self._socket = None
+                if self._debug:
+                    print('closed socket {}'.format(self._port))
+                self._closed = True
 
 
 class Bridge:
@@ -465,7 +476,7 @@ class _JavaClassFactory:
 
 class JavaObjectShadow:
     """
-    Generic class for serving as a python interface for a micromanager class using a zmq server backend
+    Generic class for serving as a python interface for a java class using a zmq server backend
     """
 
     _interfaces = (
@@ -481,20 +492,22 @@ class JavaObjectShadow:
         socket._register_java_object(self)
         self._closed = False
         # atexit.register(self._close)
+        self._close_lock = Lock()
 
     def _close(self):
-        if self._closed:
-            return
-        if not hasattr(self, "_hash_code"):
-            return  # constructor didnt properly finish, nothing to clean up on java side
-        message = {"command": "destructor", "hash-code": self._hash_code}
-        if self._bridge._debug:
-            "closing: {}".format(self)
-        self._socket.send(message)
-        reply_json = self._socket.receive()
-        if reply_json["type"] == "exception":
-            raise Exception(reply_json["value"])
-        self._closed = True
+        with self._close_lock:
+            if self._closed:
+                return
+            if not hasattr(self, "_hash_code"):
+                return  # constructor didnt properly finish, nothing to clean up on java side
+            message = {"command": "destructor", "hash-code": self._hash_code}
+            if self._bridge._debug:
+                "closing: {}".format(self)
+            self._socket.send(message)
+            reply_json = self._socket.receive()
+            if reply_json["type"] == "exception":
+                raise Exception(reply_json["value"])
+            self._closed = True
 
     def __del__(self):
         """
