@@ -598,14 +598,15 @@ class Dataset:
                     for column in column_values:
                         #remove overlap between tiles
                         if not self.has_image(**axes, **axes_to_slice, row=row, column=column):
-                            return self._empty_tile
-                        tile = self.read_image(**axes, **axes_to_slice, row=row, column=column, memmapped=True)
-                        if self.half_overlap[0] != 0:
-                            tile = tile[
-                                self.half_overlap[0] : -self.half_overlap[0],
-                                self.half_overlap[1] : -self.half_overlap[1],
-                            ]
-                        blocks[-1].append(tile)
+                            blocks[-1].append(self._empty_tile)
+                        else:
+                            tile = self.read_image(**axes, **axes_to_slice, row=row, column=column, memmapped=True)
+                            if self.half_overlap[0] != 0:
+                                tile = tile[
+                                    self.half_overlap[0] : -self.half_overlap[0],
+                                    self.half_overlap[1] : -self.half_overlap[1],
+                                ]
+                            blocks[-1].append(tile)
 
                 if rgb:
                     image = np.concatenate(
